ConstantForce

Struct ConstantForce 

pub struct ConstantForce<V> {
    pub force: V,
    pub r_0: V,
}
Apply the same force to every site, independent of the site’s properties.

The field force sets the force vector $\vec{F}$. The corresponding potential energy $U$ is:

U = - \vec{F} \cdot ( \vec{r} - \vec{r}_0 )

The vector $\vec{r}_0$ sets the reference plane where $U = 0$.

Generics

  • V: The type used to represent the position and force vectors.

Example

Basic usage:

use hoomd_interaction::external::ConstantForce;
use hoomd_vector::{Cartesian, Unit};

let constant_force = ConstantForce {
    force: Cartesian::from([0.0, -2.0]),
    r_0: Cartesian::from([0.0, -10.0]),
};

Fields

force: V

Force vector $[\mathrm{energy}] \cdot [\mathrm{length}]^{-1}$.

r_0: V

$\vec{r}_0$ $[\mathrm{length}]$: A point on the plane where $U = 0$.

Implementations

Source

impl<V> ConstantForce<V>
where V: InnerProduct,

pub fn energy(&self, r: &V) -> f64

Compute the energy of a point in a constant force field.

Example
use hoomd_interaction::external::ConstantForce;
use hoomd_vector::Cartesian;

let constant_force = ConstantForce {
    force: Cartesian::from([0.0, -2.0]),
    r_0: Cartesian::from([0.0, -10.0]),
};

let energy = constant_force.energy(&[0.0, 0.0].into());
assert_eq!(energy, 20.0);
pub fn force(&self) -> V

The force vector that acts on all sites.

Example
use hoomd_interaction::external::ConstantForce;
use hoomd_vector::Cartesian;

let constant_force = ConstantForce {
    force: Cartesian::from([0.0, -2.0]),
    r_0: Cartesian::from([0.0, -10.0]),
};

let force = constant_force.force();
assert_eq!(force, [0.0, -2.0].into());