Linear

Struct Linear 

pub struct Linear<V> {
    pub alpha: f64,
    pub plane_origin: V,
    pub plane_normal: Unit<V>,
}
Linear potential based on position.

U = \alpha \cdot \vec{n} \cdot ( \vec{r} - \vec{p} )

Computes a linear external potential at a point in space relative to the plane origin p, plane normal n, and the interaction strength alpha.

Example

Basic usage:

use hoomd_interaction::external::Linear;
use hoomd_vector::{Cartesian, Unit};

let linear = Linear {
    alpha: 2.0,
    plane_origin: [0.0, -10.0].into(),
    plane_normal: [0.0, 1.0].try_into()?,
};

Fields

alpha: f64

Interaction strength ([energy] [length]^(-1)).

plane_origin: V

Point on the plane where U=0 ([length]).

plane_normal: Unit<V>

Vector normal to the plane (unitless).

Implementations

Source

impl<V> Linear<V>
where V: InnerProduct,

pub fn energy(&self, r: &V) -> f64

Compute the energy of a point in the linear field.

Example
use hoomd_interaction::external::Linear;
use hoomd_vector::{Cartesian, Unit};

let linear = Linear {
    alpha: 2.0,
    plane_origin: [0.0, -10.0].into(),
    plane_normal: [0.0, 1.0].try_into()?,
};

let energy = linear.energy(&[0.0, 0.0].into());
assert_eq!(energy, 20.0);