pub struct Patch<V> {
pub director: Unit<V>,
pub cos_delta: f64,
}Expand description
A single patch in the AngularMask potential.
The width of the patch is given as the cosine of its half-angle.
Example
use hoomd_interaction::pairwise::angular_mask::Patch;
use std::f64::consts::PI;
let patch = Patch {
director: [0.0, 1.0, 0.0].try_into()?,
cos_delta: (PI / 4.0).cos(),
};Fields
director: Unit<V>Vector pointing from the center of the particle to the center of the mask [unitless].
cos_delta: f64Cosine of the half-angle width of the mask [unitless].