Anisotropic

Struct Anisotropic 

pub struct Anisotropic<E> {
    pub interaction: E,
    pub r_cut: f64,
}
Compute anisotropic properties from a pair of sites.

Anisotropic provides a single implementation that computes pairwise interactions that are a function of the sites’ positions and orientations. It fills the gap between traits like SitePairEnergy which operates on site properties and AnisotropicEnergy which is a function only of the relative position and orientation.

Use Anisotropic with PairwiseCutoff in MD and MC simulations.

Example

use hoomd_interaction::{
    pairwise::{AngularMask, Anisotropic, angular_mask::Patch},
    univariate::Boxcar,
};
use hoomd_vector::Angle;
use std::f64::consts::PI;

let boxcar = Boxcar {
    epsilon: -1.0,
    left: 1.0,
    right: 1.5,
};
let masks = [Patch {
    director: [1.0, 0.0].try_into()?,
    cos_delta: (PI / 8.0).cos(),
}];

let angular_mask = Anisotropic {
    interaction: AngularMask::new(boxcar, masks),
    r_cut: 1.5,
};

Fields

interaction: E

The site-site interaction.

r_cut: f64

Maximum distance between two interacting sites.