ApproximateShapeOverlap

Struct ApproximateShapeOverlap 

pub struct ApproximateShapeOverlap<E, A, B = A> {
    pub shape_i: A,
    pub shape_j: B,
    pub isotropic: E,
    pub resolution: PositiveReal,
}
Apply an isotropic potential evaluated at the approximate signed overlap distance.

Use ApproximateShapeOverlap combined with OverlapPenalty and QuickInsert to quickly insert new bodies into the microstate or QuickCompress to quickly compress the system to a target density. In both cases, ApproximateShapeOverlap will allow partial overlaps that can be resolved by later trial moves.

ApproximateShapeOverlap approximates the distance r that shape j must be moved to remove any overlaps. If the shapes are already separate, r is 0. Then it returns isotropic.energy(-r). resolution sets the steps between r values.

The overlap distance is approximate and expensive to evaluate. Do not use this potential for equilibration or sampling. It is suitable only for use during a brief initialization phase when QuickInsert is adding bodies or QuickCompress is compressing the system.

Example

use hoomd_geometry::{Convex, shape::ConvexPolygon};
use hoomd_interaction::{
    pairwise::ApproximateShapeOverlap, univariate::OverlapPenalty,
};

let approximate_shape_overlap = ApproximateShapeOverlap::new(
    Convex(ConvexPolygon::regular(6)),
    OverlapPenalty::default(),
    0.01.try_into()?,
);

Fields

shape_i: A

The site i’s shape.

shape_j: B

The site j’s shape.

isotropic: E

Potential to apply as a function of the signed distance between shapes.

resolution: PositiveReal

Resolve the separation distance in multiples of this resolution.

Implementations

Source

impl<E, G> ApproximateShapeOverlap<E, G>

pub fn new(shape: G, isotropic: E, resolution: PositiveReal) -> Self
where G: Clone,

Construct a new approximate shape overlap potential.

new() sets both shape_i and shape_j to shape. Use struct initialization syntax when you need to set these separately.

Example
use hoomd_geometry::{Convex, shape::ConvexPolygon};
use hoomd_interaction::{
    pairwise::ApproximateShapeOverlap, univariate::OverlapPenalty,
};

let approximate_shape_overlap = ApproximateShapeOverlap::new(
    Convex(ConvexPolygon::regular(6)),
    OverlapPenalty::default(),
    0.01.try_into()?,
);