Isotropic

Struct Isotropic 

pub struct Isotropic<E> {
    pub interaction: E,
    pub r_cut: f64,
}
Compute isotropic interactions between a pair of sites.

Isotropic provides a single implementation that computes pairwise interactions that are a function only of the distance between sites. It fills the gap between traits like SitePairEnergy which operates on site properties and UnivariateEnergy which is a function only of the separation distance.

Use Isotropic with PairwiseCutoff in MD and MC simulations.

Example

use hoomd_interaction::{
    SitePairEnergy, pairwise::Isotropic, univariate::LennardJones,
};
use hoomd_microstate::property::Point;
use hoomd_vector::Cartesian;

let a = Point {
    position: Cartesian::from([0.0, 0.0]),
};
let b = Point {
    position: Cartesian::from([0.0, 2.0 * 2.0_f64.powf(1.0 / 6.0)]),
};

let lennard_jones: LennardJones = LennardJones {
    epsilon: 1.5,
    sigma: 2.0,
};
let lennard_jones = Isotropic {
    interaction: lennard_jones,
    r_cut: 2.5,
};

let energy = lennard_jones.site_pair_energy(&a, &b);
assert_eq!(energy, -1.5);

Fields

interaction: E

The site-site interaction.

r_cut: f64

Maximum distance between two interacting sites.