pub trait AnisotropicEnergy<V: Vector, R: Rotate<V>> {
// Required method
fn energy(&self, r_ij: &V, o_ij: &R) -> f64;
}Expand description
Computes pairwise energies between oriented particles.
An anisotropic pairwise energy is function of the relative position and orientation of the j particle in i’s reference frame:
U(\vec{r}_{ij}, \mathbf{o}_{ij})Implement AnisotropicEnergy on a custom type or use one of the provided
potentials in pairwise in MD or MC simulations.