External

Struct External 

pub struct External<E>(pub E);
Interactions between sites and external fields.

An External newtype wrapping a type that implements SiteEnergy represents:

U_\mathrm{total} = \sum_{i=0}^{N-1} U\left( s_i \right)

where $s_i$ is the full set of site properties for site i.

An External newtype wrapping a type that implements SiteForceAndVirial and/or SiteForceVirialAndTorque represents:

\vec{F}_i = \vec{F}\left(s_i\right)
\vec{\tau}_i = \vec{\tau}\left(s_i\right)

where $\vec{F}(s_i)$ is the force computed by SiteForceAndVirial (or SiteForceVirialAndTorque) and $\vec{\tau}(s_i)$ is the torque computed by SiteForceVirialAndTorque.

A type that implements both SiteEnergy and SiteForceAndVirial (or SiteForceVirialAndTorque) must compute forces and torques that are derivatives of the energy.

Use External with ConstantForce or your own custom type that implements SiteEnergy, SiteForceAndVirial and/or SiteForceVirialAndTorque.

Examples

A linear external potential given by a constant force:

use hoomd_interaction::{External, TotalEnergy, external::ConstantForce};
use hoomd_microstate::{Body, Microstate, property::Point};
use hoomd_vector::Cartesian;

let mut microstate = Microstate::new();
microstate.extend_bodies([
    Body::point(Cartesian::from([1.0, 0.0])),
    Body::point(Cartesian::from([-1.0, 2.0])),
])?;

let constant_force = External(ConstantForce {
    force: Cartesian::from([0.0, -1.0]),
    r_0: Cartesian::default(),
});

let total_energy = constant_force.total_energy(&microstate);
assert_eq!(total_energy, 2.0);

Infinite interaction with a wall:

use hoomd_interaction::{External, SiteEnergy, TotalEnergy};
use hoomd_microstate::{Body, Microstate, property::Point};
use hoomd_vector::Cartesian;

struct Wall;

impl SiteEnergy<Point<Cartesian<2>>> for Wall {
    fn site_energy(&self, site_properties: &Point<Cartesian<2>>) -> f64 {
        if site_properties.position[1].abs() < 1.0 {
            f64::INFINITY
        } else {
            0.0
        }
    }

    fn is_only_infinite_or_zero() -> bool {
        true
    }
}

fn main() -> Result<(), Box<dyn std::error::Error>> {
    let mut microstate = Microstate::new();
    microstate.extend_bodies([
        Body::point(Cartesian::from([1.0, 1.25])),
        Body::point(Cartesian::from([-1.0, 2.0])),
    ])?;

    let wall = External(Wall);

    let total_energy = wall.total_energy(&microstate);
    assert_eq!(total_energy, 0.0);
    Ok(())
}

Tuple Fields

0: E