Rigid

Struct Rigid 

pub struct Rigid<F>(pub F);
Rigid body interactions.

The Rigid newtype implements NetBodyForceAndVirial for wrapped force interaction model types that implement NetSiteForceAndVirial. It also implements NetBodyForceVirialAndTorque for interaction model types that implement NetSiteForceVirialAndTorque.

Rigid computes the net force and torque on a rigid body that results from the forces/torques on all of its sites:

\vec{F}_\mathrm{body} = \sum_{i \in \mathrm{body}} \vec{F}_{i}
\vec{\tau}_\mathrm{body} = \sum_{i \in \mathrm{body}} (\mathbf{q}_\mathrm{body} \cdot \vec{r}_{\mathrm{body},i} \cdot \mathbf{q}_\mathrm{body}^*) \wedge \vec{F}_i + \vec{\tau}_{i}

The generic type names are:

Example

use hoomd_interaction::{
    PairwiseCutoff, Rigid, pairwise::Isotropic, univariate::LennardJones,
};

let lennard_jones: LennardJones = LennardJones {
    epsilon: 1.0,
    sigma: 1.0,
};
let evaluator = Isotropic {
    interaction: lennard_jones,
    r_cut: 2.5,
};
let rigid = Rigid(PairwiseCutoff(evaluator));

Tuple Fields

0: F