SiteForceVirialAndTorque

Trait SiteForceVirialAndTorque 

pub trait SiteForceVirialAndTorque<S> {
    type Force: Wedge + Outer;

    // Required method
    fn site_force_virial_and_torque(
        &self,
        site_properties: &S,
    ) -> (Self::Force, <Self::Force as Outer>::Tensor, <Self::Force as Wedge>::Bivector);
}
Compute the force, virial, and torque on a single site as a function of its properties.

The SiteForceVirialAndTorque trait describes a type that can compute the force, virial, and torque on a site as a function only of that site’s properties.

The external module provides a number of commonly used implementations. Combine them with External newtype for use with MD simulations.

The generic type names are:

Required Associated Types

type Force: Wedge + Outer

The type of the result force.

Required Methods

fn site_force_virial_and_torque( &self, site_properties: &S, ) -> (Self::Force, <Self::Force as Outer>::Tensor, <Self::Force as Wedge>::Bivector)

Evaluate the force, virial, and torque as a function of a single site’s properties.

Implementors