pub struct Boxcar {
pub epsilon: f64,
pub left: f64,
pub right: f64,
}Expand description
Constant valued potential in a given range of r (not differentiable).
U(r) = \begin{cases}
0 & r \lt a \\
\varepsilon & a \le r \lt b \\
0 & r \ge b
\end{cases}Compute boxcar potential function. Some uses of this in the literature call it the “square well” potential.
Examples
Basic usage:
use hoomd_interaction::univariate::{Boxcar, UnivariateEnergy};
let epsilon = 1.5;
let (left, right) = (1.0, 2.5);
let boxcar = Boxcar {
epsilon,
left,
right,
};
assert_eq!(boxcar.energy(0.0), 0.0);
assert_eq!(boxcar.energy(1.0), 1.5);
assert_eq!(boxcar.energy(2.0), 1.5);
assert_eq!(boxcar.energy(2.5), 0.0);
assert_eq!(boxcar.energy(1000.0), 0.0);The parameters are public fields and may be accessed directly:
use hoomd_interaction::univariate::{Boxcar, UnivariateEnergy};
let mut boxcar = Boxcar {
epsilon: 1.5,
left: 1.0,
right: 2.5,
};
boxcar.epsilon = -2.0;
boxcar.left = 0.0;
boxcar.right = 1.0;Fields
epsilon: f64Energy scale ([energy]).
left: f64Left side of the boxcar ([length]).
right: f64Right side of the boxcar ([length]).