pub struct Linear<V> {
pub alpha: f64,
pub plane_origin: V,
pub plane_normal: Unit<V>,
}Expand description
Linear potential based on position.
U = \alpha \cdot \vec{n} \cdot ( \vec{r} - \vec{p} )Computes a linear external potential at a point in space relative to the plane
origin p, plane normal n, and the interaction strength alpha.
Example
Basic usage:
use hoomd_interaction::external::Linear;
use hoomd_vector::{Cartesian, Unit};
let linear = Linear {
alpha: 2.0,
plane_origin: [0.0, -10.0].into(),
plane_normal: [0.0, 1.0].try_into()?,
};Fields
alpha: f64Interaction strength ([energy] [length]^(-1)).
plane_origin: VPoint on the plane where U=0 ([length]).
plane_normal: Unit<V>Vector normal to the plane (unitless).
Implementations
Sourceimpl<V> Linear<V>where
V: InnerProduct,
impl<V> Linear<V>where
V: InnerProduct,
Sourcepub fn energy(&self, r: &V) -> f64
pub fn energy(&self, r: &V) -> f64
Compute the energy of a point in the linear field.
Example
use hoomd_interaction::external::Linear;
use hoomd_vector::{Cartesian, Unit};
let linear = Linear {
alpha: 2.0,
plane_origin: [0.0, -10.0].into(),
plane_normal: [0.0, 1.0].try_into()?,
};
let energy = linear.energy(&[0.0, 0.0].into());
assert_eq!(energy, 20.0);Trait Implementations
Sourceimpl<'de, V> Deserialize<'de> for Linear<V>where
V: Deserialize<'de>,
impl<'de, V> Deserialize<'de> for Linear<V>where
V: Deserialize<'de>,
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl<S, P> SiteEnergy<S> for Linear<P>where
S: Position<Position = P>,
P: InnerProduct,
impl<S, P> SiteEnergy<S> for Linear<P>where
S: Position<Position = P>,
P: InnerProduct,
Sourcefn site_energy(&self, site_properties: &S) -> f64
fn site_energy(&self, site_properties: &S) -> f64
Evaluate the energy contribution of a single site.
Sourcefn site_energy_initial(&self, site_properties: &S) -> f64
fn site_energy_initial(&self, site_properties: &S) -> f64
Evaluate the energy contribution of a single site in the initial state. Read more
Sourcefn is_only_infinite_or_zero() -> bool
fn is_only_infinite_or_zero() -> bool
Does this potential only ever return infinity or zero? Read more
impl<V> StructuralPartialEq for Linear<V>