pub struct Isotropic<E> {
pub interaction: E,
pub r_cut: f64,
}Expand description
Compute isotropic interactions between a pair of sites.
Isotropic provides a single implementation that computes pairwise
interactions that are a function only of the distance between sites. It
fills the gap between traits like SitePairEnergy / SitePairForceAndVirial
which operate on site properties and UnivariateEnergy /
UnivariateForce which is a function only of the separation distance.
Isotropic cuts all interactions off when the distance between sites
is greater than or equal to r_cut:
U_{ij} =
\begin{cases}
U(r_{ij}) & r_{ij} < r_\mathrm{cut} \\
0 & r_{ij} \ge r_\mathrm{cut}
\end{cases}\vec{F_{ij}} =
\begin{cases}
-\frac{\mathrm{d} U}{\mathrm{d} r} \biggr\rvert_{r=r_{ji}} \hat{r}_{ji} & r_{ij} < r_\mathrm{cut} \\
\vec{0} & r_{ij} \ge r_\mathrm{cut}
\end{cases}where $U$ is given by E’s UnivariateEnergy implementation and
$-\frac{\mathrm{d} U}{\mathrm{d} r}$ is given by E’s UnivariateForce
implementation.
Use Isotropic with PairwiseCutoff in MD and MC simulations.
Example
use approxim::assert_relative_eq;
use hoomd_interaction::{
SitePairEnergy, SitePairForceAndVirial, pairwise::Isotropic,
univariate::LennardJones,
};
use hoomd_microstate::property::Point;
use hoomd_vector::Cartesian;
let a = Point {
position: Cartesian::from([0.0, 0.0]),
};
let b = Point {
position: Cartesian::from([0.0, 2.0 * 2.0_f64.powf(1.0 / 6.0)]),
};
let lennard_jones: LennardJones = LennardJones {
epsilon: 1.5,
sigma: 2.0,
};
let lennard_jones = Isotropic {
interaction: lennard_jones,
r_cut: 2.5,
};
let energy = lennard_jones.site_pair_energy(&a, &b);
let (force_ab, virial_ab) =
lennard_jones.site_pair_force_and_virial(&a, &b);
let (force_ba, virial_ba) =
lennard_jones.site_pair_force_and_virial(&b, &a);
assert_eq!(energy, -1.5);
assert_eq!(force_ab, -force_ba);
assert_relative_eq!(force_ab, Cartesian::from([0.0, 0.0]), epsilon = 1e-14);
Fields
interaction: EThe site-site interaction.
r_cut: f64Maximum distance between two interacting sites.