AnisotropicEnergy

Trait AnisotropicEnergy 

pub trait AnisotropicEnergy<V: Vector, R: Rotate<V>> {
    // Required method
    fn energy(&self, r_ij: &V, o_ij: &R) -> f64;
}
Computes pairwise energies between oriented particles.

An anisotropic pairwise energy is function of the relative position and orientation of the j particle in i’s reference frame:

U(\vec{r}_{ij}, \mathbf{o}_{ij})

Implement AnisotropicEnergy on a custom type or use one of the provided potentials in pairwise in MD or MC simulations.

Required Methods

fn energy(&self, r_ij: &V, o_ij: &R) -> f64

Compute the pairwise energy between two oriented particles.

U(\vec{r}_{ij}, \mathbf{o}_{ij})

Implementors

Source

impl<E, A, B, V, R> AnisotropicEnergy<V, R> for ApproximateShapeOverlap<E, A, B>
where E: UnivariateEnergy, V: InnerProduct, R: Rotation + Rotate<V>, A: IntersectsAt<B, V, R>,

Source

impl<E, V, R> AnisotropicEnergy<V, R> for AngularMask<E, V>
where E: UnivariateEnergy, V: InnerProduct, R: Rotate<V> + Into<R::Matrix> + Copy,