PairwiseCutoff

Struct PairwiseCutoff 

pub struct PairwiseCutoff<E>(pub E);
Short-ranged pairwise interactions between sites.

A PairwiseCutoff newtype wrapping a type that implements SitePairEnergy represents:

U_\mathrm{total} = \sum_{i=0}^{N-1}\sum_{j=i+1}^{N-1} U\left(s_i, s_j \right) \left[ \left|\vec{r}_j - \vec{r}_i\right| \lt r_\mathrm{cut} \right]\left[b_i \ne b_j\right]

where $U(s_i, s_j)$ is the potential computed by SitePairEnergy, $s_i$ is the full set of site properties for site i, $\vec{r}_i$ is the position of site i, $b_i$ is the body tag that holds site i, and $\left[ \ \right]$ denotes the Iverson bracket.

In other words, PairwiseCutoff sums the energy for all pairs that are separated by a distance less than the maximum interaction range r_cut and belong to different bodies.

A PairwiseCutoff newtype wrapping a type that implements SitePairForceAndVirial and/or SitePairForceVirialAndTorque represents:

\vec{F}_i = \sum_{j \ne i} \vec{F}\left(s_i, s_j \right) \left[ \left|\vec{r}_j - \vec{r}_i\right| \lt r_\mathrm{cut} \right]\left[b_i \ne b_j\right]
\vec{\tau}_i = \sum_{j \ne i} \vec{\tau}\left(s_i, s_j \right) \left[ \left|\vec{r}_j - \vec{r}_i\right| \lt r_\mathrm{cut} \right]\left[b_i \ne b_j\right]

where $\vec{F}(s_i, s_j)$ is the force computed by SitePairForceAndVirial (or SitePairForceVirialAndTorque) and $\vec{\tau}(s_i, s_j)$ is the torque computed by SitePairForceVirialAndTorque.

A type that implements both SitePairEnergy and SitePairForceAndVirial (or SitePairForceVirialAndTorque) must compute forces and torques that are derivatives of the energy.

Use PairwiseCutoff with Anisotropic, Isotropic, HardShape, or your own custom type that implements SitePairEnergy, SitePairForceAndVirial and/or SitePairForceVirialAndTorque.

Example

Basic usage:

use hoomd_interaction::{
    PairwiseCutoff, pairwise::Isotropic, univariate::LennardJones,
};

let lennard_jones: LennardJones = LennardJones {
    epsilon: 1.5,
    sigma: 2.0,
};
let pairwise_cutoff = PairwiseCutoff(Isotropic {
    interaction: lennard_jones,
    r_cut: 5.0,
});

Set a custom potential using a closure (implements only SitePairEnergy):

use hoomd_interaction::{PairwiseCutoff, pairwise::Isotropic};

let pairwise_cutoff = PairwiseCutoff(Isotropic {
    interaction: |r: f64| 1.0 / (r.powi(12)),
    r_cut: 3.0,
});

Implement a custom potential via a type:

use hoomd_interaction::{
    PairwiseCutoff, pairwise::Isotropic, univariate::UnivariateEnergy,
};

struct Custom {
    a: f64,
}

impl UnivariateEnergy for Custom {
    fn energy(&self, r: f64) -> f64 {
        self.a / r.powi(12)
    }
}

let custom = Custom { a: 2.0 };
let pairwise_cutoff = PairwiseCutoff(Isotropic {
    interaction: custom,
    r_cut: 2.0,
});

Hard sphere:

use hoomd_interaction::{PairwiseCutoff, pairwise::HardSphere};
use hoomd_microstate::property::Point;
use hoomd_vector::Cartesian;

let hard_sphere = PairwiseCutoff(HardSphere { diameter: 1.0 });

Hard ellipse:

use hoomd_geometry::shape::Ellipse;
use hoomd_interaction::{PairwiseCutoff, pairwise::HardShape};
use hoomd_microstate::property::Point;
use hoomd_vector::Cartesian;
let ellipse = Ellipse::with_semi_axes([4.0.try_into()?, 1.0.try_into()?]);
let hard_ellipse = PairwiseCutoff(HardShape(ellipse));

Tuple Fields

0: E

Implementations

Source

impl<E> PairwiseCutoff<E>

pub fn site_pair_force_and_virial<V, S>( &self, site_i: &Site<S>, site_j: &Site<S>, ) -> (V, V::Tensor)
where E: SitePairForceAndVirial<S, Force = V>, V: Default + Outer, V::Tensor: Default,

Calculate the pairwise force and virial on site i caused by site j.

Use this method to compute an individual term in the net force and virial on site i, subject to the the maximum interaction range r_cut and inter-body checks:

\begin{align*}
\vec{F}_{i} &= \sum_{j \in N_s} \vec{F}_{ji}
\mathbf{W}_{i} &= \sum_{j \in N_s} \mathbf{W}_{ji}
\end{align*}

where $N_s$ is the set of neighboring sites in other bodies for which $\left|\vec{r}_j - \vec{r}_i\right| \lt r_\mathrm{cut}$ and the subscript $ji$ means “from j on i”.

Example
use approxim::assert_relative_eq;
use hoomd_interaction::{
    PairwiseCutoff, pairwise::Isotropic, univariate::LennardJones,
};
use hoomd_linear_algebra::matrix::Matrix;
use hoomd_microstate::{Body, Microstate, Site, property::Point};
use hoomd_vector::Cartesian;

let mut microstate = Microstate::new();
microstate.extend_bodies([
    Body::point(Cartesian::from([0.0, 0.0, 0.0])),
    Body::point(Cartesian::from([1.0, 0.0, 0.0])),
])?;

let lennard_jones: LennardJones = LennardJones {
    epsilon: 1.0,
    sigma: 1.0,
};

let force = PairwiseCutoff(Isotropic {
    interaction: lennard_jones,
    r_cut: 2.5,
});

let sites = microstate.sites();
let (force_0, virial_0) =
    force.site_pair_force_and_virial(&sites[0], &sites[1]);
let (force_1, virial_1) =
    force.site_pair_force_and_virial(&sites[1], &sites[0]);

assert_relative_eq!(force_0, Cartesian::from([-24.0, 0.0, 0.0]));
assert_eq!(
    virial_0,
    Matrix {
        rows: [[12.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
    }
);

assert_relative_eq!(force_1, Cartesian::from([24.0, 0.0, 0.0]));
assert_eq!(
    virial_1,
    Matrix {
        rows: [[12.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
    }
);
pub fn site_pair_force_virial_and_torque<V, S>( &self, site_i: &Site<S>, site_j: &Site<S>, ) -> (V, V::Tensor, V::Bivector)
where E: SitePairForceVirialAndTorque<S, Force = V>, V: Default + Wedge + Outer, V::Bivector: Default, V::Tensor: Default,

Calculate the pairwise force, virial, and torque on site i caused by site j.

Use this method to compute an individual term in the net force, virial, and torque on site i, subject to the the maximum interaction range r_cut and inter-body checks:

\begin{align*}
\vec{F}_{i} &= \sum_{j \in N_s} \vec{F}_{ji} \\
\mathbf{W}_{i} &= \sum_{j \in N_s} \mathbf{W}_{ji} \\
\vec{\tau}_{i} &= \sum_{j \in N_s} \vec{\tau}_{ji} \\
\end{align*}

where $N_s$ is the set of neighboring sites in other bodies for which $\left|\vec{r}_j - \vec{r}_i\right| \lt r_\mathrm{cut}$ and the subscript $ji$ means “from j on i”.

pub fn site_pair_energy<S>(&self, site_i: &Site<S>, site_j: &Site<S>) -> f64
where E: SitePairEnergy<S>,

Compute the pair energy between two sites.

Use this method to compute an individual term in the total pair energy, subject to the the maximum interaction range r_cut and inter-body checks:

U\left(s_i, s_j \right) \left[ \left|\vec{r}_j - \vec{r}_i\right| \lt r_\mathrm{cut} \right]\left[b_i \ne b_j\right]
Example
use approxim::assert_relative_eq;
use hoomd_interaction::{
    PairwiseCutoff, pairwise::Isotropic, univariate::LennardJones,
};
use hoomd_microstate::{Body, Microstate, Site};
use hoomd_vector::Cartesian;

let lennard_jones: LennardJones = LennardJones {
    epsilon: 1.0,
    sigma: 1.0,
};
let pairwise_cutoff = PairwiseCutoff(Isotropic {
    interaction: lennard_jones,
    r_cut: 2.5,
});

let body_a = Body::point(Cartesian::from([0.0, 0.0]));
let body_b = Body::point(Cartesian::from([0.0, 3.0]));
let body_c = Body::point(Cartesian::from([0.0, -2.0_f64.powf(1.0 / 6.0)]));

let microstate = Microstate::builder()
    .bodies([body_a, body_b, body_c])
    .try_build()?;

let sites = microstate.sites();
let energy_ab = pairwise_cutoff.site_pair_energy(&sites[0], &sites[1]);
let energy_ac = pairwise_cutoff.site_pair_energy(&sites[0], &sites[2]);

assert_eq!(energy_ab, 0.0);
assert_relative_eq!(energy_ac, -1.0);