pub trait NetBodyForceVirialAndTorque<B, S, X, C> {
type Force: Wedge + Outer;
// Required method
fn net_body_force_virial_and_torque(
&self,
microstate: &Microstate<B, S, X, C>,
body_index: usize,
) -> (Self::Force, <Self::Force as Outer>::Tensor, <Self::Force as Wedge>::Bivector);
}Expand description
Sum all the forces, virials, and torques that act on a given body in a microstate.
See Rigid for an example.
The generic type names are:
B: TheBody::propertiestype.S: TheSite::propertiestype.X: The spatial data structure type.C: Theboundarycondition type.
Required Associated Types
Required Methods
Sourcefn net_body_force_virial_and_torque(
&self,
microstate: &Microstate<B, S, X, C>,
body_index: usize,
) -> (Self::Force, <Self::Force as Outer>::Tensor, <Self::Force as Wedge>::Bivector)
fn net_body_force_virial_and_torque( &self, microstate: &Microstate<B, S, X, C>, body_index: usize, ) -> (Self::Force, <Self::Force as Outer>::Tensor, <Self::Force as Wedge>::Bivector)
Compute the net force, virial, and torque on a body in the microstate.
microstate describes the configuration and body_index is the index
of the body to compute.
Returns: TODO