pub trait SitePairForceVirialAndTorque<S> {
type Force: Wedge + Outer;
// Required method
fn site_pair_force_virial_and_torque(
&self,
site_properties_i: &S,
site_properties_j: &S,
) -> (Self::Force, <Self::Force as Outer>::Tensor, <Self::Force as Wedge>::Bivector);
}Expand description
Compute the pairwise force, virial, and torque on one site from another site.
The SitePairForceVirialAndTorque trait describes a type that can compute
the force, virial, and torque on a site by another site as a function of
the two sites’ properties.
The pairwise module provides a number of commonly used implementations.
Combine them with PairwiseCutoff newtype for use with MD simulations.
The generic type names are:
S: TheSite::propertiestype.