pub struct Harmonic {
pub k: f64,
pub r_0: f64,
}Expand description
Quadratic potential well centered at a given separation distance.
U = \frac{1}{2} k (r - r_0)^2Compute the harmonic potential and force as a function of r with
equilibrium spring length r_0.
Examples
use approxim::{assert_abs_diff_eq, assert_relative_eq};
use hoomd_interaction::univariate::{
Harmonic, UnivariateEnergy, UnivariateForce,
};
let k = 2.0;
let r_0 = 0.0;
let harmonic = Harmonic { k, r_0 };
assert_abs_diff_eq!(harmonic.energy(0.0), 0.0);
assert_relative_eq!(harmonic.energy(1.0), 1.0);
assert_abs_diff_eq!(harmonic.force(1.0), -2.0, epsilon = 1e-12);The parameters are public fields and may be accessed directly:
use hoomd_interaction::univariate::Harmonic;
let mut harmonic = Harmonic { k: 1.0, r_0: 0.0 };
harmonic.k = 5.0;
harmonic.r_0 = 1.0;Fields
k: f64Spring constant $[\mathrm{energy}] [\mathrm{length}]^{-2}$.
r_0: f64Equilibrium spring length $[\mathrm{length}]$.