pub struct LennardJones<const N: i32 = 12, const M: i32 = 6> {
pub epsilon: f64,
pub sigma: f64,
}Expand description
Potential with a steep repulsive core and an attractive well.
U(r) = 4 \varepsilon \left[ \left( \frac{\sigma}{r} \right)^{N} - \left( \frac{\sigma}{r} \right)^{M} \right]Compute the Lennard-Jones (LJ) potential and force as a function of r.
Examples
In basic usage, the exponents N and M default to 12 and 6, respectively:
use approxim::{assert_abs_diff_eq, assert_relative_eq};
use hoomd_interaction::univariate::{
LennardJones, UnivariateEnergy, UnivariateForce,
};
let epsilon = 1.5;
let sigma = 2.5;
let lennard_jones: LennardJones = LennardJones { epsilon, sigma };
assert_abs_diff_eq!(lennard_jones.energy(sigma), 0.0);
assert_relative_eq!(
lennard_jones.energy(2.0_f64.powf(1.0 / 6.0) * sigma),
-epsilon
);
assert_abs_diff_eq!(
lennard_jones.force(2.0_f64.powf(1.0 / 6.0) * sigma),
0.0,
epsilon = 1e-12
);You can choose any values for N and M at compile time:
use approxim::{assert_abs_diff_eq, assert_relative_eq};
use hoomd_interaction::univariate::{
LennardJones, UnivariateEnergy, UnivariateForce,
};
let epsilon = 1.5;
let sigma = 2.5;
let lennard_jones: LennardJones<8, 4> = LennardJones { epsilon, sigma };
assert_abs_diff_eq!(lennard_jones.energy(sigma), 0.0);
assert_relative_eq!(
lennard_jones.energy(2.0_f64.powf(1.0 / 4.0) * sigma),
-epsilon
);
assert_abs_diff_eq!(
lennard_jones.force(2.0_f64.powf(1.0 / 4.0) * sigma),
0.0,
epsilon = 1e-12
);The parameters are public fields and may be accessed directly:
use hoomd_interaction::univariate::LennardJones;
let mut lennard_jones: LennardJones = LennardJones::default();
lennard_jones.epsilon = 1.5;
lennard_jones.sigma = 3.0;Fields
epsilon: f64Energy scale ([energy]).
sigma: f64Interaction width ([length]).